#include <MFRC522.h>

MFRC522 rfid(10, 9);
MFRC522::MIFARE_Key key = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
MFRC522::StatusCode status;
char buffer[49];

void setup() {
  Serial.begin(9600);
  SPI.begin();
  rfid.PCD_Init();
  Serial.println(F("Avvicina la tessera RFID"));
}

void loop() {
  if (rfid.PICC_IsNewCardPresent() && rfid.PICC_ReadCardSerial()) {
    Serial.println(F("Lettura in corso..."));
    if (readCard(buffer)) {
      Serial.println(F("Contenuto della tessera RFID:"));
      Serial.println(buffer);
    }

    rfid.PICC_HaltA();
    rfid.PCD_StopCrypto1();

    Serial.println();
    Serial.println(F("Avvicina la tessera RFID"));
  }
}

bool readCard(char *content) {
    int index = 0;
    byte blockData[18];

    for (byte blockOffset = 0; blockOffset < 3; blockOffset++) {
      memset(blockData, '\0', 18);
      if (!readBlock(4 + blockOffset, blockData)) {
        return false;
      }

      for (int i = 0; i < 16; i++) {
        content[index++] = blockData[i];
        if (blockData[i] == '\0') {
          return true;
        }
      }
    }

    content[index] = '\0';
    return true;
}

bool readBlock(byte blockAddr, byte *blockData) {
  byte length = 18;

  status = rfid.PCD_Authenticate(
    MFRC522::PICC_CMD_MF_AUTH_KEY_A, 
    blockAddr, 
    &key, 
    &(rfid.uid)
  );
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Autenticazione del blocco non riuscita: "));
    Serial.println(blockAddr);
    Serial.println(rfid.GetStatusCodeName(status));
    return false;
  }

  status = rfid.MIFARE_Read(blockAddr, blockData, &length);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Errore durante la lettura del blocco: "));
    Serial.println(blockAddr);
    Serial.println(rfid.GetStatusCodeName(status));
    return false;
  }

  return true;
}
